#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "types.h"
#include "proto.h"
#include "matrix.h"
#include "select.h"
#include "print.h"
#include "chisq.h"
#include "Box-test.h"

/* local prototypes */
static real m_value(Matrix * mat, const real partimean[]); 

/* local macros */
#define Square(X) ((X) * (X))

/* module specific global variables */
static const double         alpha_level      = 0.05;


/*                                                                         *\
   -----------------------------------------------------------------------
                     Box test for equality of variance
   -----------------------------------------------------------------------
\*                                                                         */

real m_value(      Matrix *   mat,
             const real       partimean[])
{
   register matrix_dim parti, query;
            matrix_dim maxparti = num_cols(mat);
            matrix_dim maxquery = num_rows(mat);
   register real       temp;
            real       numer,
                       denom;

   numer = denom = 0.0;
   for (parti=0; parti < maxparti; parti++) {
      for (query=0; query < maxquery; query++) {
         temp = mat_value(mat, query, parti) - partimean[parti];
         temp *= temp;
         numer += Square(temp);
         denom += temp;
      }
   }
   denom *= denom;
   numer *= (maxparti * maxquery);
   return((numer/denom) - 3.0);
}/* m_value() */


bool G_test(      Matrix * mat,
            const real     partimean[],
                  real     partivar[],
            const bool     showall) 
{
/*
    Return TRUE iff the null hypothesis (that the variances are equal) is 
                    accepted.

    From Supplement 4A `A Robust Test of the Equality of Several Variances'
    (pp. 167-8) of _Intermediate Business Statistics: Analysis of Variance,
    Regression, and Time Series_ by Robert B. Miller and Dean W. Wichern;
    Published by Holt, Rinehart and Winston ((c) 1977) in the Series in
    Quantitative Method for Decision Making.

    N.B.: Should not be used for sample sizes smaller than 10.
*/
          const    char       proc[] = "G_test";
                   matrix_dim maxparti = num_cols(mat);
                   matrix_dim maxquery = num_rows(mat);
          const    matrix_dim min_sample_size = 10;
          register matrix_dim parti;
          register double     sum;
                   double     chi;
                   real       ksquared,
                              m,
                              nbar;
                   real       constant[5];
   extern          char       errorstring[];
   extern          int        errno;

   if (showall) {
      printf("   Box test: ");
      NewLine(1, stdout);
      if (maxparti < min_sample_size) {
         printf("    Warning: there are only "MAT_DIM_SPEC" samples.  ", \
                maxparti);
         printf("The Box test is not ");
         NewLine(1, stdout);
         printf("             recommended for sample sizes less than "   \
                MAT_DIM_SPEC".", min_sample_size);
         NewLine(1, stdout);
      }
   }
   nbar = maxquery;
   m = m_value(mat, partimean);
   ksquared = 2.0 + m - ( m / nbar);
   constant[0] = (real)maxquery - 1.0;          /* n_j - 1 */
   constant[1] = constant[0] * (real)maxparti;  
   for (constant[2]=0.0,parti=0; parti < maxparti; parti++) {
      errno = 0;
      constant[2] += (constant[0] * log(partivar[parti]));
      if (0 != errno) {
         sprintf(errorstring, "log(partivar["ELEM_SPEC"] = "REAL_SPEC")", \
                              parti, partivar[parti]);
         swerr(proc, errorstring, 0);
      }
   }
   constant[3] = constant[0] / ksquared;  /* not used anymore */
   constant[4] = constant[2] / constant[1];

   for (sum=0.0,parti=0; parti < maxparti; parti++) {
      register real temp;

      errno = 0;
      temp = log(partivar[parti]) - constant[4];
      if (0 != errno) {
         sprintf(errorstring, "log(partivar["ELEM_SPEC"] = "REAL_SPEC")", \
                              parti, partivar[parti]);
         swerr(proc, errorstring, 0);
      }
      temp = Square(temp);
      sum += (constant[0] * temp);
   }
   sum /= ksquared;
   chi = critchi((double)alpha_level, (int)(maxparti));
   if (showall) {
      printf("     m   ="POS_REAL_SPEC, m);
      NewLine(1, stdout);
      printf("     k^2 ="POS_REAL_SPEC, ksquared);
      NewLine(1, stdout);
      printf("     G   ="POS_REAL_SPEC, sum);
      NewLine(1, stdout);
      printf("     "                                                    \
             "chi squared["MAT_DIM_SPEC", alpha=%.2f] = "POS_REAL_SPEC, \
             maxparti, alpha_level, chi);
      NewLine(1, stdout);
      printf("   "POS_REAL_SPEC" %s "POS_REAL_SPEC, \
             sum, (sum > chi)?">":"<=", chi);
      NewLine(1, stdout);
      printf("   %significant difference detected.", (sum <= chi)?"No s":"S");
      NewLine(2, stdout);
   }
   return (sum <= chi);
} /* G_test() */

