/* Hartley.c */

#include <stdio.h>
#include "types.h"
#ifdef UWOCSD
   #include "proto.h"
#endif
#include "Hartley.h"
#include "HartleyTable.h"
#include "print.h"

/* local prototypes */
static real Fmax_value(const matrix_dim dof, const matrix_dim points, 
           const bool showall);


bool Hartley_test(const matrix_dim numobs,   /* how many observations */
                  const matrix_dim dof,      /* degrees of freedom */
                  const real       value,    /* critical value to compare 
                                                   against table */
                  const bool       showall)
{
   const string suffix="assuming homoscedasticity";
         char   prefix[35] = "";
         bool   okay = TRUE;

         sprintf(prefix, "Hartley test(c="MAT_DIM_SPEC", df="MAT_DIM_SPEC \
                         ", alpha=0.050): ",                              \
                          numobs, dof);

   if (dof < Fmaxmin_dof) {
      printf("%snot enough degrees of freedom", prefix);
      okay = FALSE;
   } else 
   if (numobs < min_H_samples) {
      printf("%snot enough observations", prefix);
      okay = FALSE;
   } else 
   if (numobs > max_H_samples) {
      printf("%stoo many degrees of freedom", prefix);
      okay = FALSE;
   } 

   if (! okay) {
      NewLine(1, stdout);
      printf("%*s%s", (int)strlen(prefix), "", suffix);
      NewLine(1, stdout);
      return(TRUE);
   } else {
      if (showall) {
         real F = Fmax_value(numobs, dof, showall);

         printf("   "REAL_SPEC" %s "REAL_SPEC, \
                value, (value > F)?">":"<=", F);
         NewLine(1, stdout);
      }
      return(value > Fmax_value(numobs, dof, FALSE));
   }
} /* Hartley_test() */


real Fmax_value(const matrix_dim dof,    /*degrees of freedom */
                const matrix_dim points, /*# of independent mean squares*/
                const bool       showall)
{
/*
   Returns critical value of Hartley's F_max test for alpha = 0.05.
   Uses linear interpolation for values that are not in table.
*/
   short i;
   real  result;
      
   for(i=0; (i < Fmaxmax_dof) && (dof > Fmaxdof[i]); i++)
      /* empty */ ;

   if (Fmaxmax_dof == i) {  /* oo */
      result = Fmax[i - 1][points - min_H_samples];
      if (showall) {
         printf("   Fmax[dof=oo;c="MAT_DIM_SPEC"] = "POS_REAL_SPEC, \
                points - min_H_samples, result);
         NewLine(1, stdout);
      }
      return(result);
   } else 
   if (Fmaxdof[i] == dof) { /* in table as itself */
      result = Fmax[i][points - min_H_samples];
      if (showall) {
         printf("   Fmax[df="MAT_DIM_SPEC", num. samples="MAT_DIM_SPEC \
                ", alpha=0.05] = "POS_REAL_SPEC,                       \
                points, dof, result);
         NewLine(1, stdout);
      }
      return(result);
   } else {
      /* linear interpolation */
      real diff=((real)(dof - Fmaxdof[i-1]) /      \
                 (real)(Fmaxdof[i] - Fmaxdof[i-1]));

      result = (Fmax[i]  [points - min_H_samples] + \
                Fmax[i-1][points - min_H_samples]) * diff;
      if (showall) {
         printf("   Fmax[dof="MAT_DIM_SPEC";c="MAT_DIM_SPEC"] approx. = " \
                POS_REAL_SPEC,                                            \
                dof, points, result);
         NewLine(1, stdout);
      }
      return(result);
   }
} /* Fmax_value() */


