
/* boolarr.c */

/**************************************************************************

  Boolean Vectors in C
  
  FILENAME: boolarr.c
  LANGUAGE: ANSI C
  REQUIRES: "types.h" (see below for minimal contents)

  AUTHOR:   Jamie Blustein <jamie@csd.uwo.ca>/<jamie@uwo.ca>
  CREATED:  8 April 1995 from bitarr.c
  MODIFIED: 8 April 1995

  DESCRIPTION: Functions to create and manipulate arrays of boolean values.
     Functions to: dynamically create arrays, access (read and write)
     elements; convert from bit vectors to strings.
               It is the caller's responsibility to know the size of the
     bit vector.  One way to keep track of the size is to wrap the 
     vector in a wrapper like the one below.  Note that the // is used to
     mark a single line comment.  Only the first two items are necessary
     but the others are included for illustration.

      typedef struct {
                elem_t  size;     // how many items in selected
                bits *  selected; // bit vector recording which 
                                  // elements are selected 
                elem_t  max;      // maximum possible size
                char ** name;     // array of names of items 
                elem_t  max_len;  // maximum possible length of a name 
                char *  title;    // what data is represented 
                                  // by this struct? 
            } chose_t;

 ***************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include "types.h"
#include "boolarr.h" /* exported prototypes */
#ifdef UWOCSD
int fputs(char * s, FILE * stream);
#endif

/*                                                                  *\
   ----------------------------------------------------------------
                  Initialization and Creation Code
   ----------------------------------------------------------------
\*                                                                  */

elem_t ba_init(void) 
{
   return(1);
} /* ba_init() */


bit *ba_new(const elem_t nelems) 
{
/*
   ba_new()
   
   PURPOSE: dynamically allocate space for an array of `nelems' bits
         and initalize the bits to all be zero.
   PRE:  nelems is the number of Boolean values required in an array
   POST: either a pointer to an initialized (all zero) array of bit 
      OR
         space was not available and NULL was returned
   NOTE: calloc() guarantees that the space has been initialized to 0.

   Used by: ba_ul2b(), ba_intersection() and ba_union().
*/

   return ((bit *)calloc(nelems, sizeof(bit)));
} /* ba_new() */


/*                                                                 *\
   ---------------------------------------------------------------
                   Assigning and Retrieving Values
   ---------------------------------------------------------------
\*                                                                 */
void ba_assign(      bit    arr[],
                     elem_t elem,
               const bool   value) 
{
/*
   ba_assign()

   PURPOSE: set or clear the bit in position `elem' of the array
         `arr'
   PRE:     arr[elem] is to be set (assigned to 1) if value is TRUE, 
            otherwise it is to be cleared (assigned to 0).  
   POST:    PRE fulfilled.  All other bits unchanged.
   SEE ALSO: ba_all_assign()
   Used by:  ba_ul2b()
*/

   arr[elem] = value;
} /* ba_assign() */



bool ba_value(const bit    arr[],
              const elem_t elem) 
{
/*
   ba_value()

   PRE:  arr must have at least elem elements
   POST: The value of the `elem'th element of arr has been returned
         (as though `arr' was just a 1-dimensional array of bit)
   Used by: ba_b2str() and ba_count()
*/

   return(TRUE == arr[elem]);
} /* ba_value() */



void ba_toggle(      bit     arr[],
               const elem_t  elem) 
{
/*
   ba_toggle()

   PRE:  arr must have at least elem elements
   POST: The value of the `elem'th element of arr has been flipped, 
         i.e. if it was 1 it is 0; if it was 0 it is 1.
   SEE ALSO: ba_complement()
*/

 arr[elem] = !arr[elem];
} /* ba_toggle() */



void ba_all_assign(      bit    arr[],
                   const elem_t size,
                   const bool   value) 
{
/*
   ba_all_assign()

   PRE:  arr has been initialized to have *exactly* size elements.
   POST: All `size' elements of arr have been set to `value'.  
         The array is in canonical form, i.e. trailing elements are 
         all 0.
   NOTE: The array allocated by ba_new() has all elements 0 and is 
         therefore in canonical form.
   SEE ALSO: ba_assign()
   Used by: ba_ul2b()
*/
   register elem_t i;

   for (i=0; i < size; i++) {
      arr[i] = value;
   }
} /* ba_all_assign() */


/*                                                                  *\
   ----------------------------------------------------------------
                         Conversion Routines
   ----------------------------------------------------------------
\*                                                                  */

char * ba_b2str(const bit    arr[],
                const elem_t size,
                      char * dest)
{
/*
   ba_b2str()

   PRE: `arr' is a bit array with at least `size' elements.  Either
         `dest' points to enough allocated space to hold `size' + 1
         characters or `dest' is NULL and such space is to be
         dynamically allocated.
   POST: Either `dest' points to a null-terminated string that
         contains a character representation of the first `size'
         elements of the bit array `arr';
      OR 
         `dest' is NULL and a request to dynamically allocate memory
         for a string to hold a character representation of `arr' was
         not be granted.
   Used by: ba_print()
*/
   register elem_t i;

   if ((NULL != dest) || \
       (NULL != (dest = (char *)malloc(size + 1)))) {

      for (i=0; i < size; i++) {
         dest[i] = (ba_value(arr,i) ?'1' :'0');
      }
      dest[size] = '\0';
   }
   return (dest);
} /* ba_b2str() */


/*                                                                  *\
   ----------------------------------------------------------------
                      Mathematical Applications
   ---------------------------------------------------------------- 
\*                                                                  */

unsigned long ba_count(const bit    arr[],
                       const elem_t size) 
{
/*
   ba_count()

   PRE:  `arr' is an allocated bit array with at least `size'
         elements
   POST: The number of 1 bits in the first `size' elements of `arr'
         have been returned.
   NOTE: if arr is not in canonical form, i.e. if some unused bits
         are 1, then an unexpected value may be returned.
*/

  register unsigned long count; 
  register elem_t        i;     

  for (count = 0L, i = 0; i < size; i++) {
     if (ba_value(arr, i)) {
        count++;
     }
  }
  return (count);
} /* ba_count() */


bool ba_print(const bit    arr[],
              const elem_t size,
                    FILE * dest) 
{
   char * to_print = ba_b2str(arr, size, NULL);

   if (NULL != to_print) {
      bool status = (EOF != fputs(to_print, dest));
      free(to_print);
      return (status);
   } else {
      return (FALSE);
   }
} /* ba_print() */
